%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Estimate the parameters of a sample for the Generalized Logistic
% Distribution, using the L-moments method. The Hosking and Wallis (1997) 
% version of the distribution is chosen. 
%
% Input :
%    -x : vector of values
%   
% Output : 
%   -xi (location), alpha (scale), k (shape) : parameters of the distribution
%
% If k > 0, the range of x is : -Inf < x < xi + alpha/k
% If k = 0, the range of x is : -Inf < x < Inf 
% If k < 0, the range of x is : xi + alpha/k <= x < Inf 
%
% If k = 0 , the distribution corresponds to the logistic distribution
%
% Source : Hosking, J., & Wallis, J. (1997). Regional Frequency Analysis:
% An Approach Based on L-Moments. Cambridge: Cambridge University Press. 
% doi:10.1017/CBO9780511529443
%
% Guillaume Talbot, INRS-ETE 2021
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [xi,alpha,k] = glo_fit(x)

%Remove the missing values
x(isnan(x))=[];

%Number of non-missing values in the vector
n=length(x);
     
%Sort the vector x
x=sort(x);

j=1:n;
if size(j)~=size(x) %If j and x have different orientations, we transpose j
    j=j';
end

%Compute the L-moments : l1, l2, l3, l4, t, t3 and t4
b0=sum(x)/n;
b1j=(j-1)./(n-1).*x;
b2j=(j-1).*(j-2)./(n-1)./(n-2).*x;
%b3j=(j-1).*(j-2).*(j-3)./(n-1)./(n-2)./(n-3).*x; %Not necessary for this distribution

b1=sum(b1j)/n;
b2=sum(b2j)/n;
%b3=sum(b3j)/n; %Not necessary for this distribution

l1=b0;
l2=2*b1-b0;
l3=6*b2-6*b1+b0;
%l4=20*b3-30*b2+12*b1-b0;  %Not necessary for this distribution

%t=l2/l1; %Not necessary for this distribution
t3=l3/l2;
%t4=l4/l2; %Not necessary for this distribution


%Compute the parameters using the L-moments
k=-t3;
alpha=l2*sin(pi*k)/pi/k;
xi=l1-alpha*((1/k)-(pi/sin(k*pi)));
